@@ -15,7 +15,7 @@ if not motd_text then return; end
 
 local st = require "util.stanza";
 
-motd_text = motd_text:gsub("^%s*(.-)%s*$", "%1"):gsub("\n%s+", "\n"); -- Strip indentation from the config
+motd_text = motd_text:gsub("^%s*(.-)%s*$", "%1"):gsub("\n[ \t]+", "\n"); -- Strip indentation from the config
 
 module:hook("presence/bare", function (event)
 		local session, stanza = event.origin, event.stanza;
